"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PermissionsService = void 0;
const Base_1 = require("./Base");
const Permissions_1 = require("./Permissions");
class PermissionsService extends Base_1.Base {
    constructor(lensPlatformClient) {
        super(lensPlatformClient);
        this.permissions = new Permissions_1.Permissions();
    }
    /**
     * Clarifies whether a given user can perform an action in a given space.
     * @param action - `Actions` enum value
     * @param forSpace - Space object that must contain `{ teams: Team[] }`
     * @param forUserId - string userId, defaults to the id of current access token bearer
     * @param forRevokeInvitation - Additional information to determine if a user can revoke invitation
     * @returns boolean
     * @throws "Could not get role for space with no teams" exception
     */
    canSpace(action, forSpace, forUserId, forRevokeInvitation) {
        return this.permissions.canSpace(action, forSpace, forUserId, forRevokeInvitation);
    }
    /**
     * Clarifies whether a given user can execute the specified Team action
     * @param action - 'TeamActions' enum value
     * @param space - Space object that must contain `{ teams: Team[] }`
     * @param team - Team object
     * @param forUserId - User that is executing the action
     * @param targetUserId - User that is the target of the exection, e.g. user to be removed
     * @returns
     */
    // eslint-disable-next-line max-params
    canTeam(action, space, team, forUserId, targetUserId) {
        return this.permissions.canTeam(action, space, team, forUserId, targetUserId);
    }
    /**
     * Clarifies whether a given user can perform an action for a given K8sCluster.
     * @param action - `Actions` enum value
     * @param forSpace - Space object that must contain `{ teams: Team[] }`
     * @param forK8sCluster - K8sCluster
     * @param forUserId - string userId
     * @returns boolean
     * @throws "Could not get role for space with no teams" exception
     */
    canK8sCluster(action, forSpace, forK8sCluster, forUserId) {
        return this.permissions.canK8sCluster(action, forSpace, forK8sCluster, forUserId);
    }
    /**
     * DEPRECATED. WILL BE REMOVED.
     * Clarifies whether a given user can perform an action in a given space.
     * @param action - `Actions` enum value
     * @param forSpace - Space object that must contain `{ teams: Team[] }`
     * @param forUserId - string userId, defaults to the id of current access token bearer
     * @returns boolean
     * @throws "Could not get role for space with no teams" exception
     * @deprecated Use .canSpace instead.
     */
    canI(action, forSpace, forUserId) {
        return this.canSpace(action, forSpace, forUserId);
    }
    /**
     * Gets a role the user with specified user Id has in specified `space`.
     * @param space - Space object that must contain `{ teams: Team[] }`
     * @param forUserId - string userId, defaults to the id of current access token bearer
     * @returns Role enum value
     * @throws "Could not get role for space with no teams" exception
     */
    getRole(space, forUserId) {
        return this.permissions.getRole(space, forUserId);
    }
}
exports.PermissionsService = PermissionsService;
