"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserRolesService = void 0;
const Base_1 = require("./Base");
const exceptions_1 = require("./exceptions");
const Permissions_1 = require("./Permissions");
const rolesToTeamEntityKindMap = {
    [Permissions_1.Roles.Owner]: "Owner",
    [Permissions_1.Roles.Admin]: "Admin",
    [Permissions_1.Roles.Member]: "Normal",
};
const teamEntityKindToRolesMap = {
    ["Owner"]: Permissions_1.Roles.Owner,
    ["Admin"]: Permissions_1.Roles.Admin,
    ["Normal"]: Permissions_1.Roles.Member,
};
const getNotFoundException = (error, userName) => {
    if (!error) {
        return new exceptions_1.NotFoundException();
    }
    const message = error === null || error === void 0 ? void 0 : error.body.message;
    if (typeof message === "string" && message.includes("Space not found")) {
        return new exceptions_1.SpaceNotFoundException();
    }
    return new exceptions_1.UserNameNotFoundException(userName);
};
class UserRolesService extends Base_1.Base {
    async getUserSpaceRole(spaceName, userName) {
        const { apiEndpointAddress, fetch } = this.lensPlatformClient;
        const url = `${apiEndpointAddress}/spaces/${spaceName}/users/${userName}/role`;
        const json = await (0, exceptions_1.throwExpected)(async () => fetch.get(url), {
            404: (error) => getNotFoundException(error, userName),
        });
        const result = json;
        // TeamEntityKind on client are Roles. Map TeamEntityKind to Roles.
        return {
            role: teamEntityKindToRolesMap[result.role],
        };
    }
    async setUserRole(spaceName, userName, role) {
        const { apiEndpointAddress, fetch } = this.lensPlatformClient;
        const url = `${apiEndpointAddress}/spaces/${spaceName}/users/${userName}/role`;
        // Roles on the API side are TeamEntityKind. Map Roles to TeamEntityKind.
        const teamRole = rolesToTeamEntityKindMap[role];
        const json = await (0, exceptions_1.throwExpected)(async () => fetch.patch(url, { role: teamRole }), {
            404: (error) => getNotFoundException(error, userName),
            422(error) {
                const message = error === null || error === void 0 ? void 0 : error.body.message;
                if (typeof message === "string" && message.includes("Role should be")) {
                    return new exceptions_1.InvalidRoleException(role);
                }
                if (typeof message === "string" && message.includes("User is already assign")) {
                    return new exceptions_1.RoleAlreadyAssignedException(role, userName);
                }
                return new exceptions_1.UnprocessableEntityException();
            },
            500: () => new exceptions_1.InternalServerException(),
        });
        const result = json;
        // TeamEntityKind on client are Roles. Map TeamEntityKind to Roles.
        return {
            role: teamEntityKindToRolesMap[result.role],
        };
    }
}
exports.UserRolesService = UserRolesService;
