import { Base } from "./Base";
import type { Business, SubscriptionCustomField, UsedSeat, UserBusinessRole } from "./BusinessService";
import { BillingPageToken, License } from "./types/types";
export type UserAttribute = {
    id: string;
    userId: string;
    value: string;
    name: string;
};
/**
 *
 * @remarks
 * This interface should be generated using OpenAPI generator in the future.
 *
 * @alpha
 */
export interface User {
    id?: string;
    username?: string;
    fullname?: string;
    firstName?: string;
    lastName?: string;
    userAttributes?: Array<UserAttribute>;
}
export type UserWithEmail = User & {
    email: string;
};
export interface UserAttributes {
    fullname?: string;
    company?: string;
    tshirt?: string;
}
export interface UserCommunications {
    marketing?: boolean;
    onboarding?: boolean;
}
export type SubscriptionState = "active" | "canceled" | "expired" | "failed" | "future" | "paused";
export type UserBusinessWithSSOInfo = {
    role: UserBusinessRole;
} & Pick<Business, "name" | "handle"> & {
    sso: {
        id: string;
        identityProviderID: string;
    } | null;
};
export type OfflineActivationCode = {
    activationCode: string;
};
export type SubscriptionInfo = {
    id?: string | null;
    planName?: string | null;
    planCode?: string | null;
    /**
     * Current payment period start date string
     */
    currentPeriodStartedAt?: string | null;
    /**
     * Current payment period end date string
     */
    currentPeriodEndsAt?: string | null;
    /**
     * Trial start date string
     */
    trialStartedAt?: string | null;
    /**
     * Trial end date string
     */
    trialEndsAt?: string | null;
    /**
     * Total number of seats in this subscription. (Recurly subscription["quantity"])
     */
    seats: number;
    /**
     * The subscription that have been assigned to a user (`user_subscriptions` relation)
     */
    usedSeats: UsedSeat[];
    /**
     * Custom fields stored for the subscription
     */
    customFields?: SubscriptionCustomField[];
    /**
     * Name of the company of the Recurly account
     */
    companyName?: string | null;
    /**
     * Account code of the subscription's Recurly account
     */
    accountCode?: string | null;
    autoRenew: boolean;
    collectionMethod: "manual" | "automatic";
    /**
     * Subscription seat used for offline
     */
    offline: boolean;
    /**
     * True if the subscription belongs to a business Recurly account
     */
    isBusinessAccount: boolean;
    /**
     * State of the subscription
     */
    state: SubscriptionState;
};
export type SubscriptionSeat = {
    id?: string | null;
    planName?: string | null;
    planCode?: string | null;
    /**
     * Current payment period start date string
     */
    currentPeriodStartedAt?: string | null;
    /**
     * Current payment period end date string
     */
    currentPeriodEndsAt?: string | null;
    /**
     * Trial start date string
     */
    trialStartedAt?: string | null;
    /**
     * Trial end date string
     */
    trialEndsAt?: string | null;
    /**
     * Total number of seats in this subscription. (Recurly subscription["quantity"])
     */
    seats: number;
    /**
     * The subscription that have been assigned to a user (`user_subscriptions` relation)
     */
    usedSeats: UsedSeat[];
    /**
     * Name of the company of the Recurly account
     */
    companyName?: string | null;
    /**
     * Account code of the subscription's Recurly account
     */
    accountCode?: string | null;
    /**
     * True if the subscription belongs to a business Recurly account
     */
    isBusinessAccount: boolean;
    /**
     * State of the Recurly subscription
     */
    recurlySubscriptionState: SubscriptionState;
    /**
     * Subscription activation date string
     */
    activatedAt: string;
    /**
     * Subscription deactivation date string
     */
    deactivatedAt: string | null;
    /**
     * Subscription creation date string
     */
    createdAt: string;
    /**
     * Subscription update date string
     */
    updatedAt: string;
    /**
     * Subscription expiration date string
     */
    expiredAt: string | null;
    /**
     * Recurly subscription UUID format ID
     */
    subscriptionId: string;
    /**
     * Recurly subscription short format ID
     */
    shortSubscriptionId: string;
    /**
     * Is the seat active
     */
    active: boolean;
    /**
     * Subscription used offline
     */
    offline: boolean;
};
export type Address = {
    /**
     * Phone number
     */
    phone: string | null;
    /**
     * Street 1
     */
    street1: string | null;
    /**
     * Street 2
     */
    street2: string | null;
    /**
     * City
     */
    city: string | null;
    /**
     * State or province.
     */
    region: string | null;
    /**
     * Zip or postal code.
     */
    postalCode: string | null;
    /**
     * Country, 2-letter ISO 3166-1 alpha-2 code.
     */
    country: string | null;
};
export type BillingInfo = {
    lastName: string | null;
    firstName: string | null;
    company: string | null;
    address: Address | null;
    paymentMethod: {
        cardType: string | null;
        firstSix: string | null;
        lastTwo: string | null;
        lastFour: string | null;
        expMonth: Number | null;
        expYear: Number | null;
    };
};
export interface ActivationCodeData {
    accessToken: string;
    idToken: string;
    refreshToken: string;
}
export type UploadAvatarResponse = {
    uri: string;
};
export interface Invoice {
    date: Date | null;
    number: string | null;
    planCode: string | null;
    total: number | null;
    subtotal: number | null;
    state: string | null;
    currency: string | null;
    billingInfo: BillingInfo | null;
    /**
     * string if the invoice is from a child account of a LBID
     */
    fromChildBusinessId?: Business["id"];
}
export interface LinkedUserAccount {
    identityProviderAlias: string | undefined;
    identityProviderDisplayName: string | undefined;
    username: string | undefined;
}
/**
 *
 * The class for consuming all `user` resources.
 *
 * currently we have:
 * - `.getOne` maps to `GET /users/{username}`
 * - `.getBillingPageToken` maps to `GET /users/{username}/billing-page-token`
 *
 * @remarks
 * This class should be generated using OpenAPI in the future.
 *
 * @alpha
 */
declare class UserService extends Base {
    getUsername(): Promise<string>;
    getOne({ username }: {
        username: string;
    }, queryString?: string): Promise<UserWithEmail>;
    getMany(queryString?: string): Promise<User[]>;
    /**
     * Update user
     */
    updateOne(username: string, user: User & {
        attributes?: UserAttributes;
    } & {
        password?: string;
    } & {
        email?: string;
    }): Promise<User>;
    /**
     * Reset user password
     */
    resetPassword(username: string, password: string): Promise<void>;
    getSelf(): Promise<UserWithEmail>;
    deleteOne(username: string): Promise<void>;
    getUserSubscriptions(username: string): Promise<SubscriptionInfo[]>;
    getUserSubscriptionsSeats(username: string): Promise<SubscriptionSeat[]>;
    activateSubscription({ username, license, }: {
        username: string;
        license: License;
    }): Promise<License>;
    activateSubscriptionSeat({ username, license, }: {
        username: string;
        license: License;
    }): Promise<License>;
    getSubscriptionSeatOfflineActivationCode({ username, subscriptionSeatId, activationCodeData, }: {
        username: string;
        subscriptionSeatId: string;
        activationCodeData: ActivationCodeData;
    }): Promise<OfflineActivationCode | null>;
    deactivateSubscriptionSeat({ username, license, }: {
        username: string;
        license: Pick<License, "subscriptionId">;
    }): Promise<void>;
    getBillingPageToken(username: string): Promise<BillingPageToken>;
    getBillingPageTokenBySubscriptionId(username: string, subscriptionId: string): Promise<BillingPageToken>;
    getUserBillingInformation(username: string): Promise<BillingInfo>;
    getUserInvoices(username: string): Promise<Invoice[]>;
    /**
     * Get all emails (primary+secondary) of the user
     */
    getEmails(): Promise<Record<string, "verified" | "unverified" | "primary">>;
    /**
     * Get user linked accounts
     */
    getUserLinkedAccounts(): Promise<LinkedUserAccount[]>;
    /**
     * Get user businesses
     */
    getUserBusinesses(): Promise<UserBusinessWithSSOInfo[]>;
    /**
     * Get user communication preferences
     */
    getUserCommunicationsPreferences(): Promise<UserCommunications>;
    /**
     * Update user communication preferences
     */
    updateUserCommunicationsPreferences(communicationPreferences: UserCommunications): Promise<UserCommunications>;
    /**
     * Delete user linked account
     */
    deleteUserLinkedAccount(identityProviderAlias: string): Promise<void>;
    /**
     * Add new secondary email(s) to user's account, newly added email will be unverified
     * and can be promoted to primary later verification.
     */
    createEmails(emails: string[]): Promise<string[]>;
    /**
     * Fetch the avatar image as base64-encoded string
     * @param username
     */
    getAvatar(username: string): Promise<string>;
    /**
     * Upload user avatar
     * @param avatar: HTML form file input value
     */
    uploadAvatar(avatar: File): Promise<UploadAvatarResponse>;
    /**
     * Delete secondary email(s) from user account
     */
    deleteEmails(emails: string[]): Promise<void>;
    /**
     * Send a verification email to a user's unverified secondary email address
     */
    sendVerificationEmail(email: string): Promise<void>;
    /**
     * Verify a user's secondary email using the token
     */
    verifySecondaryEmail(token: string): Promise<void>;
    /**
     * Set a verified email as user's primary email
     */
    setPrimaryEmail(email: string): Promise<void>;
    confirmPersonalLicenseEligibility(username: string): Promise<void>;
    getUserFullName(user: User): string;
}
export { UserService };
