import { OpenIdConnect } from "./OpenIdConnect";
import { UserService } from "./UserService";
import { SpaceService } from "./SpaceService";
import { TeamService } from "./TeamService";
import { PermissionsService } from "./PermissionsService";
import { InvitationService } from "./InvitationService";
import { PlanService } from "./PlanService";
import { BillingPageTokenService } from "./BillingPageTokenService";
import { BusinessService } from "./BusinessService";
import { NotificationService } from "./NotificationService";
import axios from "axios";
import pino from "pino";
import decode from "jwt-decode";
import { UserRolesService } from "./UserRolesService";
import { LensDesktopKubeService } from "./LensDesktopKubeService";
import { SSOService } from "./SSOService";
// Axios defaults to xhr adapter if XMLHttpRequest is available.
// LensPlatformClient supports using the http adapter if httpAdapter
// option is set to true
// eslint-disable-next-line @typescript-eslint/no-require-imports, @typescript-eslint/no-var-requires
let axiosHttpAdapter;
// lens-platform-sdk works in multiple environments, namely
// * Node: regular Node.js process and electron's Node.js
// * Browser: Regular browsers and Electron's Chromium environment
// For this reason we need to support different kind of requires
// In particular we might want to load the http adapter to execute
// requests in Node.
const multiEnvironmentRequire = (moduleName) => {
    try {
        return window.require(moduleName);
    }
    catch (error) {
        return require(moduleName);
    }
};
/**
 * Returns axios http adapter (used for Node.js requests).
 * It requires the adapter if not loaded before.
 * @returns
 */
const getAxiosHttpAdapter = () => {
    try {
        if (!axiosHttpAdapter) {
            axiosHttpAdapter = multiEnvironmentRequire("axios").getAdapter("http");
        }
        return axiosHttpAdapter;
    }
    catch (error) {
        return undefined;
    }
};
const requestLibraryMethods = [
    "get",
    "post",
    "put",
    "patch",
    "head",
    "delete",
];
/**
 * Function to determine if func is a function of the request library for making a request
 */
const isRequestLibraryFunction = (func, key) => typeof func === "function" && requestLibraryMethods.includes(key);
class LensPlatformClient {
    constructor(options) {
        var _a, _b;
        if (!options) {
            throw new Error(`Options can not be ${options}`);
        }
        const { accessToken, getAccessToken, httpAdapter, proxyConfigs, httpAgent, httpsAgent } = options;
        if (!accessToken && !getAccessToken) {
            throw new Error(`Both accessToken ${accessToken} or getAccessToken are ${getAccessToken}`);
        }
        this.logLevel = (_a = options.logLevel) !== null && _a !== void 0 ? _a : "silent";
        this.logger = pino({ level: this.logLevel });
        this.accessToken = accessToken;
        this.httpAdapter = httpAdapter;
        this.proxyConfigs = proxyConfigs;
        this.httpAgent = httpAgent;
        this.httpsAgent = httpsAgent;
        this.getAccessToken = getAccessToken;
        this.keyCloakAddress = options.keyCloakAddress;
        this.keycloakRealm = options.keycloakRealm;
        this.apiEndpointAddress = options.apiEndpointAddress;
        this.defaultHeaders = (_b = options.defaultHeaders) !== null && _b !== void 0 ? _b : {};
        this.user = new UserService(this);
        this.lensDesktopKube = new LensDesktopKubeService(this);
        this.space = new SpaceService(this);
        this.roles = new UserRolesService(this);
        this.team = new TeamService(this);
        this.plan = new PlanService(this);
        this.permission = new PermissionsService(this);
        this.invitation = new InvitationService(this);
        this.billingPageToken = new BillingPageTokenService(this);
        this.business = new BusinessService(this);
        this.sso = new SSOService(this);
        this.openIDConnect = new OpenIdConnect(this);
        this.notification = new NotificationService(this);
    }
    async getToken(url) {
        const token = this.getAccessToken && typeof this.getAccessToken === "function"
            ? await this.getAccessToken(url)
            : this.accessToken;
        return token;
    }
    async getDecodedAccessToken() {
        const token = await this.getToken();
        if (token) {
            return decode(token);
        }
        return undefined;
    }
    get authHeader() {
        return {
            Authorization: `Bearer ${this.accessToken}`,
        };
    }
    /**
     * A proxied version of request library that
     *
     * 1) Prints request/response in console (for developer to debug issues)
     * 2) Auto add `Authorization: Bearer [token]`
     *
     */
    get fetch() {
        const getToken = this.getToken.bind(this);
        const { defaultHeaders, httpAdapter, logger, proxyConfigs, httpAgent, httpsAgent } = this;
        const proxy = new Proxy(axios, {
            get(target, key) {
                const prop = target[key];
                if (isRequestLibraryFunction(prop, key)) {
                    return async (...arg) => {
                        var _a;
                        try {
                            const url = arg[0];
                            // "get", "head", "delete" has options in the second parameter
                            // "patch", "post", "put" has the options in the third parameter
                            const hasBody = !["get", "head", "delete"].includes(key);
                            const options = hasBody ? arg[2] : arg[1];
                            const requestBody = hasBody ? arg[1] : null;
                            const headers = options === null || options === void 0 ? void 0 : options.headers;
                            let restOptions;
                            if (headers) {
                                const clone = { ...options };
                                delete clone.headers;
                                restOptions = clone;
                            }
                            else {
                                restOptions = options;
                            }
                            // Print HTTP request info in developer console
                            logger.debug(`${key === null || key === void 0 ? void 0 : key.toUpperCase()} ${url}`);
                            let token;
                            if (!(restOptions === null || restOptions === void 0 ? void 0 : restOptions.unauthenticated)) {
                                token = await getToken(url);
                            }
                            const requestHeaders = {
                                ...(token ? { Authorization: `Bearer ${token}` } : {}),
                                ...headers,
                                ...defaultHeaders,
                            };
                            const requestOptions = {
                                headers: requestHeaders,
                                ...(httpAdapter ? { adapter: getAxiosHttpAdapter() } : {}),
                                ...(proxyConfigs ? { proxy: proxyConfigs } : {}),
                                ...(httpAgent ? { httpAgent } : {}),
                                ...(httpsAgent ? { httpsAgent } : {}),
                                // Merge options
                                ...restOptions,
                            };
                            logger.debug(`request arguments ${JSON.stringify(requestOptions)}`);
                            const response = await (hasBody
                                ? prop(url, requestBody, requestOptions)
                                : prop(url, requestOptions));
                            // Body as JavaScript plain object
                            const body = response.data;
                            // Print HTTP response info in developer console
                            logger.debug(`${key === null || key === void 0 ? void 0 : key.toUpperCase()} ${response === null || response === void 0 ? void 0 : response.status} ${response === null || response === void 0 ? void 0 : response.statusText} ${url} `);
                            logger.debug(`response body: ${body}`);
                            return body;
                        }
                        catch (error) {
                            // @ts-expect-error
                            logger.error(`error message: ${error === null || error === void 0 ? void 0 : error.message}`);
                            // @ts-expect-error
                            logger.error(`error response body: ${(_a = error === null || error === void 0 ? void 0 : error.response) === null || _a === void 0 ? void 0 : _a.body}`);
                            // eslint-disable-next-line @typescript-eslint/no-throw-literal
                            throw error;
                        }
                    };
                }
                return prop;
            },
        });
        return proxy;
    }
}
export default LensPlatformClient;
