import { Base } from "./Base";
import { InternalServerException, InvalidRoleException, NotFoundException, RoleAlreadyAssignedException, SpaceNotFoundException, throwExpected, UnprocessableEntityException, UserNameNotFoundException, } from "./exceptions";
import { Roles } from "./Permissions";
const rolesToTeamEntityKindMap = {
    [Roles.Owner]: "Owner",
    [Roles.Admin]: "Admin",
    [Roles.Member]: "Normal",
};
const teamEntityKindToRolesMap = {
    ["Owner"]: Roles.Owner,
    ["Admin"]: Roles.Admin,
    ["Normal"]: Roles.Member,
};
const getNotFoundException = (error, userName) => {
    if (!error) {
        return new NotFoundException();
    }
    const message = error === null || error === void 0 ? void 0 : error.body.message;
    if (typeof message === "string" && message.includes("Space not found")) {
        return new SpaceNotFoundException();
    }
    return new UserNameNotFoundException(userName);
};
class UserRolesService extends Base {
    async getUserSpaceRole(spaceName, userName) {
        const { apiEndpointAddress, fetch } = this.lensPlatformClient;
        const url = `${apiEndpointAddress}/spaces/${spaceName}/users/${userName}/role`;
        const json = await throwExpected(async () => fetch.get(url), {
            404: (error) => getNotFoundException(error, userName),
        });
        const result = json;
        // TeamEntityKind on client are Roles. Map TeamEntityKind to Roles.
        return {
            role: teamEntityKindToRolesMap[result.role],
        };
    }
    async setUserRole(spaceName, userName, role) {
        const { apiEndpointAddress, fetch } = this.lensPlatformClient;
        const url = `${apiEndpointAddress}/spaces/${spaceName}/users/${userName}/role`;
        // Roles on the API side are TeamEntityKind. Map Roles to TeamEntityKind.
        const teamRole = rolesToTeamEntityKindMap[role];
        const json = await throwExpected(async () => fetch.patch(url, { role: teamRole }), {
            404: (error) => getNotFoundException(error, userName),
            422(error) {
                const message = error === null || error === void 0 ? void 0 : error.body.message;
                if (typeof message === "string" && message.includes("Role should be")) {
                    return new InvalidRoleException(role);
                }
                if (typeof message === "string" && message.includes("User is already assign")) {
                    return new RoleAlreadyAssignedException(role, userName);
                }
                return new UnprocessableEntityException();
            },
            500: () => new InternalServerException(),
        });
        const result = json;
        // TeamEntityKind on client are Roles. Map TeamEntityKind to Roles.
        return {
            role: teamEntityKindToRolesMap[result.role],
        };
    }
}
export { UserRolesService };
